from query_handler import get_event_gmap_info

if __name__ == '__main__':
    temp = get_event_gmap_info('上個月17號三峽發生火災')
    try:
        print(temp['data'])
    except TypeError:
        print(temp)

    temp = get_event_gmap_info('5月12號台中市的北屯區東光路550之11號，有回收廠的大量廢棄物發生了火警')
    try:
        print(temp['data'])
    except TypeError:
        print(temp)

    temp = get_event_gmap_info('三峽發生火災')
    try:
        print(temp['data'])
    except TypeError:
        print(temp)

    temp = get_event_gmap_info('三峽昨天發生火災')
    try:
        print(temp['data'])
    except TypeError:
        print(temp)


    temp = get_event_gmap_info('昨天下午三點三峽發生火災')
    try:
        print(temp['data'])
    except TypeError:
        print(temp)


    temp = get_event_gmap_info('5月12號清晨六點左右，台中市的北屯區東光路550之11號，有回收廠的大量廢棄物發生了火警')
    try:
        print(temp['data'])
    except TypeError:

        print(temp)

    temp = get_event_gmap_info('現在三峽發生火災')
    try:
        print(temp['data'])
    except TypeError:
        print(temp)

    temp = get_event_gmap_info('上個月17號下午三點三峽發生火災')
    try:
        print(temp['data'])
    except TypeError:
        print(temp)

    # error, error, error, error, success, success, success, success


import datetime as dt
import json
from .config import OPENAI_KEY, GOOGLE_API_KEY, DEFAULT_PROMPT
#from openai_utils import openai_init
from .gmap_utils import get_gmap_url
import openai

def get_event_gmap_info(str_input, lang='CH', **kwargs):
    """
    解析字串中的事件、時間和地點，並生成相應的Google地圖URL。

    參數:
        str_input (str): 要解析的文本。
        lang (str, 可選): 預設為 'CH'。指定回覆的語言，可以是 'CH'（中文）或 'EN'（英文）。
        **kwargs: 其他選項性參數。特別地，可以使用 'prompt_path' 參數來指定提示訊息文檔的路徑。

    讀取prompt_path檔案時的注意事項:
        格式: 請確保提詞文檔的格式是正確的，並且可以被JSON解析。
        必須欄位:
            time: 須為YYYY-mm-dd HH:MM:SS的格式
            location: 地點描述，例如"台北車站"
            event: 事件說明，例如"環保會議"
        清晰度: 提詞應該清楚明瞭，以避免模型產生歧義或不準確的回應。
        可解析: 確保提詞和模型回傳的內容都是可被JSON解析的，避免包含可能破壞JSON格式的特殊字符。

    回傳:
        dict: 包含以下鍵值：
            - 'time': 事件時間（格式為 'YYYY-MM-DD HH:MM:SS'）。
            - 'location': 事件地點。
            - 'event': 事件描述。
            - 'gmap': Google地圖的URL。
    例外:
        ValueError: 如果指定了不支援的語言。
    
    使用方式:
        result = get_event_gmap_info("包含事件、時間與地點的字串", lang='EN', prompt_path="your_prompt_path.txt")
    """
    if lang not in ['EN', 'CH']:
        raise ValueError('supported language: CH, EN')
        
    now = dt.datetime.utcnow() + dt.timedelta(hours=8)
    weekday = ['Monday','Tuesday','Wednesday','Thursday','Friday','Saturday','Sunday']
    
    #openai_init(OPENAI_KEY)

    prompt_path = kwargs.get('prompt_path', None)

    if prompt_path:
        with open(prompt_path, 'r', encoding='utf-8') as f:
            str_prompt = f.read()
    else:
        str_prompt = DEFAULT_PROMPT
    str_prompt = f'# This is your known information:\n「現在」(now) is defined as {now.strftime("%Y-%m-%d %H:%M:%S")}，{weekday[now.isoweekday()-1]}\n' + str_prompt + ('Please reply in English' if lang=='EN' else '請用中文回覆我')
    prompt = []
    prompt.append({
        'role': 'system',
        'content': str_prompt
    })
    prompt.append({
        'role': 'user',
        'content': f'{str_input}'
    })
    model = 'gpt-4-1106-preview'
    client = openai.OpenAI(api_key=OPENAI_KEY)
    response = client.chat.completions.create(model=model,
                                            messages=prompt,
                                            max_tokens=300)
    content = response.choices[-1].message.content
    try:
        result = json.loads(content)
        if result['status'] == 'error':
            result['data']['lat'] = None
            result['data']['lon'] = None
            result['data']['gmap'] = ''
            return result
        lat, lon, gmap_url = get_gmap_url(result['data']['location'], GOOGLE_API_KEY)
        result['data']['lat'] = lat
        result['data']['lon'] = lon
        result['data']['gmap'] = gmap_url
        return result
    except json.decoder.JSONDecodeError:
        print(content, flush=True)
        return "抱歉, 我無法解析您的請求, 請試試換句話說" if lang == 'CH' else 'Sorry, I cannot recognize your sentence, please ask in another sentence.'



import os
from dotenv import load_dotenv

load_dotenv(dotenv_path='./.env')

OPENAI_KEY = os.getenv('OPENAI_KEY')
GOOGLE_API_KEY = os.getenv('GOOGLE_API_KEY')
DEFAULT_PROMPT = '''
# You are very proficient in parsing the time, location, and events from sentences.
You serve a program designed by a company outsourced by the Taiwan government's environmental protection department, which requires you to parse the time, location, and event from sentences, and respond in 'a single' json format, no more than one, no need for any content or inquiries other than json. 
If included, it will cause errors in parsing the json program, so do not add the json format.
Example as follows: {"status":"{status (success or error)}", "message":"{empty string or error message}", "data":{"time":"{YYYY-mm-dd HH:MM:SS}","location":"{location}","event":"{what happened}"}}.
Replace the text in {} in the example and remove {}.
Remove milliseconds from time.

# For relative time expressions like "now", "last month" etc. It should be directly parsed as the specific time point you have defined.
# If there is only year missing, use this year.
# If user mention N o'clock, use H:00:00 as time.
# If year<1000, it's 民國年. Just add 1911 into it.

# In the parsing process, if a specific time point cannot be identified, such as only mentioning the date without time, the status should be set to error and leave data.time empty.

# Provide clear instructions in Taiwanese customary language in the message to reduce the occurrence of parsing errors.
# If the parsing is successful, then fill in the corresponding time, location, and event information in the respective fields of the json.
# The guidance provided to users in cases where specific time information is missing should be clear and helpful.
'''

if OPENAI_KEY is None:
    raise EnvironmentError("未設置OPENAI_KEY環境變數。請檢查您的.env文件或環境設置。")

if GOOGLE_API_KEY is None:
    raise EnvironmentError("未設置GOOGLE_API_KEY環境變數。請檢查您的.env文件或環境設置。")



import geocoder

def get_gmap_url(location, google_key):
    location_data = geocoder.google(location, key=google_key)
    if location_data.status != 'OK':
        return None, None, f'抱歉, Google查詢經緯度的程式遇到了{location_data.status}問題, 但可以提供你如下參考網址: https://www.google.com/maps?q={location}'
    location_data = location_data.geojson['features'][0]['properties']
    return location_data["lat"], location_data["lng"], f'https://www.google.com.tw/maps/search/{location_data["lat"]},{location_data["lng"]}'


import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="cameo_eco_query",
    version="1.0.10",
    author="JcXGTcW",
    author_email="jcxgtcw@cameo.tw",
    description="cameo_eco_query",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bohachu/cameo-eco-query",  # 維持原本的repo URL
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'openai==1.6.1',
        'python-dotenv>=1.0.0',
        'geocoder>=1.38.1'
    ],
)
